/****************************************************
  Projekt: TMCM-ARM (TMCL fr ARM-Module)

  Modul:   TMC262-ARM.c
           Zugriffsfunktionen fr den TMC262

  Datum:   12.11.2009 OK
*****************************************************/


#include "bits.h"
#include "at91sam7x256.h"
#include <stdlib.h>

#include "Eval429.h"
#include "Eval429-SPI.h"
#include "Eval429-TMC262.h"
#include "Eval429-TMC428.h"
#include "Eval429-Systimer.h"


//Datenstrukturen fr Softwarekopien der TMC262-Register
typedef struct
{
  UCHAR Intpol;
  UCHAR DEdge;
  UCHAR MRes;
} TStepDirConfig;

typedef struct
{
  UCHAR BlankTime;
  UCHAR ChopperMode;
  UCHAR HysteresisDecay;
  UCHAR RandomTOff;
  UCHAR HysteresisEnd;
  UCHAR HysteresisStart;
  UCHAR TOff;
  UCHAR DisableFlag;
} TChopperConfig;

typedef struct
{
  UCHAR SmartIMin;
  UCHAR SmartDownStep;
  UCHAR SmartStallLevelMax;
  UCHAR SmartUpStep;
  UCHAR SmartStallLevelMin;
} TSmartEnergyControl;

typedef struct
{
  UCHAR FilterEnable;
  signed char StallGuardThreshold;
  UCHAR CurrentScale;
} TStallGuardConfig;


typedef struct
{
  UCHAR TestMode;
  UCHAR SlopeHighSide;
  UCHAR SlopeLowSide;
  UCHAR ProtectionDisable;
  UCHAR ProtectionTimer;
  UCHAR StepDirectionDisable;
  UCHAR VSenseScale;
  UCHAR ReadBackSelect;
} TDriverConfig;


typedef enum {RW_SDCONF, RW_CHOPCONF, RW_SECTRL, RW_SGCONF, RW_DRVCONF} TReadWriteConfig;


//Variablen
static TStepDirConfig StepDirConfig[N_O_MOTORS];           //Softwarekopien der DRVCTRL-Register aller Treiberchips
static TChopperConfig ChopperConfig[N_O_MOTORS];           //Softwarekopien der CHOPCONF-Register aller Treiberchips
static TSmartEnergyControl SmartEnergyControl[N_O_MOTORS]; //Softwarekopien der SMARTEN-Register aller Treiberchips
static TStallGuardConfig StallGuardConfig[N_O_MOTORS];     //Softwarekopien der SGSCONF-Register aller Treiberchips
static TDriverConfig DriverConfig[N_O_MOTORS];             //Softwarekopien der DRVCONF-Register aller Treiberchips
static TReadWriteConfig ReadWriteConfig[N_O_MOTORS];       //gibt an, welches Konfigurationstelegramm gerade zum Auslesen verwendet wird
static UINT SPIReadInt;
static UINT SPIWriteInt;

static UINT SPIStepDirConf;
static UINT SPIChopperConf;
static UINT SPISmartConf;
static UINT SPISGConf;
static UINT SPIDriverConf;

static UCHAR WaitingForDatagram;
static UINT CoverDatagramDelay;


//Funktionen

/*******************************************************************
   Funktion: ReadWrite262()
   Parameter: Which428: Auswahl des TMC262 bei Modulen mit mehr als 1 Achse
              Read: Zeiger auf Integer fr das Ergebnis
              Write: Array mit drei Bytes die in den TMC262
                     hineingeschrieben werden sollen.

   Rckgabewert: ---

   Zweck: SPI-Kommunikation mit dem TMC262. Die Telegramme werden gleich
          an die richtigen Positionen geschoben.
********************************************************************/
static void ReadWrite262(UCHAR Which262, UINT *ReadInt, UINT WriteInt)
{
  switch(Which262)
  {
    case 0:
      if(IS_SD_MODE_SELECTED())
      {
        *ReadInt=ReadWriteSPI(SPI_DEV_TMC262, WriteInt >> 16, FALSE);
        *ReadInt<<=8;
        *ReadInt|=ReadWriteSPI(SPI_DEV_TMC262, WriteInt >> 8, FALSE);
        *ReadInt<<=8;
        *ReadInt|=ReadWriteSPI(SPI_DEV_TMC262, WriteInt & 0xff, TRUE);
        *ReadInt>>=4;
      }
      else
      {
        Write428Int(IDX_COVER_DATA, WriteInt);
        while(Read428Status() & TMC428_STATUS_CDGW);
        *ReadInt=Read428Int(IDX_LOW_WORD);
      }
      break;

    case 1:
      *ReadInt=ReadWriteSPI(SPI_DEV_TMC261, WriteInt >> 16, FALSE);
      *ReadInt<<=8;
      *ReadInt|=ReadWriteSPI(SPI_DEV_TMC261, WriteInt >> 8, FALSE);
      *ReadInt<<=8;
      *ReadInt|=ReadWriteSPI(SPI_DEV_TMC261, WriteInt & 0xff, TRUE);
      *ReadInt>>=4;
      break;

    case 2:
      *ReadInt=ReadWriteSPI(SPI_DEV_TMC260, WriteInt >> 16, FALSE);
      *ReadInt<<=8;
      *ReadInt|=ReadWriteSPI(SPI_DEV_TMC260, WriteInt >> 8, FALSE);
      *ReadInt<<=8;
      *ReadInt|=ReadWriteSPI(SPI_DEV_TMC260, WriteInt & 0xff, TRUE);
      *ReadInt>>=4;
      break;
  }
}


/*******************************************************************
   Funktion: WriteStepDirConfig()
   Parameter: Which262: Auswahl des TMC262 bei Modulen mit mehr als 1 Achse

   Rckgabewert: ---

   Zweck: Beschreiben des DRVCTRL-Registers des TMC262 im Step/Dir-Modus.
          Die Parameeter mssen in der globalen Variablen StepDirConfig
          stehen.
********************************************************************/
static void WriteStepDirConfig(UCHAR Which262)
{
  SPIWriteInt=0;
  if(StepDirConfig[Which262].Intpol) SPIWriteInt|=BIT9;
  if(StepDirConfig[Which262].DEdge) SPIWriteInt|=BIT8;
  if(StepDirConfig[Which262].MRes>15) StepDirConfig[Which262].MRes=15;
  SPIWriteInt|=StepDirConfig[Which262].MRes;

  ReadWrite262(Which262, &SPIReadInt, SPIWriteInt);
  SPIStepDirConf=SPIWriteInt;
}


/*******************************************************************
   Funktion: WriteChopperConfig()
   Parameter: Which262: Auswahl des TMC262 bei Modulen mit mehr als 1 Achse

   Rckgabewert: ---

   Zweck: Beschreiben des CHOPCONF-Registers des TMC262.
          Die Parameter mssen in der globalen Variablen ChopperConfig
          stehen.
********************************************************************/
static void WriteChopperConfig(UCHAR Which262)
{
  if(ChopperConfig[Which262].BlankTime>3) ChopperConfig[Which262].BlankTime=3;
  if(ChopperConfig[Which262].HysteresisDecay>3) ChopperConfig[Which262].HysteresisDecay=3;
  if(ChopperConfig[Which262].HysteresisEnd>15) ChopperConfig[Which262].HysteresisEnd=15;
  if(ChopperConfig[Which262].HysteresisStart>7) ChopperConfig[Which262].HysteresisStart=7;
  if(ChopperConfig[Which262].TOff>15) ChopperConfig[Which262].TOff=15;

  SPIWriteInt=0;
  SPIWriteInt|=BIT19;  //Registeraddresse CHOPCONF;
  SPIWriteInt|=((UINT) ChopperConfig[Which262].BlankTime) << 15;
  if(ChopperConfig[Which262].ChopperMode) SPIWriteInt|=BIT14;
  if(ChopperConfig[Which262].RandomTOff) SPIWriteInt|=BIT13;
  SPIWriteInt|=((UINT) ChopperConfig[Which262].HysteresisDecay) << 11;
  SPIWriteInt|=((UINT) ChopperConfig[Which262].HysteresisEnd) << 7;
  SPIWriteInt|=((UINT) ChopperConfig[Which262].HysteresisStart) << 4;
  if(!ChopperConfig[Which262].DisableFlag) SPIWriteInt|=((UINT) ChopperConfig[Which262].TOff);  //wenn DisableFlag gesetzt wird 0 gesendet

  ReadWrite262(Which262, &SPIReadInt, SPIWriteInt);
  SPIChopperConf=SPIWriteInt;
}


/*******************************************************************
   Funktion: WriteSmartEnergyControl()
   Parameter: Which262: Auswahl des TMC262 bei Modulen mit mehr als 1 Achse

   Rckgabewert: ---

   Zweck: Beschreiben des SMARTEN-Registers des TMC262.
          Die Parameter mssen in der globalen Variablen SmartEnergyControl
          stehen.
********************************************************************/
static void WriteSmartEnergyControl(UCHAR Which262)
{
  if(SmartEnergyControl[Which262].SmartIMin>1) SmartEnergyControl[Which262].SmartIMin=1;
  if(SmartEnergyControl[Which262].SmartDownStep>3) SmartEnergyControl[Which262].SmartDownStep=3;
  if(SmartEnergyControl[Which262].SmartStallLevelMax>15) SmartEnergyControl[Which262].SmartStallLevelMax=15;
  if(SmartEnergyControl[Which262].SmartUpStep>3) SmartEnergyControl[Which262].SmartUpStep=3;
  if(SmartEnergyControl[Which262].SmartStallLevelMin>15) SmartEnergyControl[Which262].SmartStallLevelMin=15;

  SPIWriteInt=0;
  SPIWriteInt|=BIT19|BIT17;  //Registeradresse SMARTEN
  SPIWriteInt|=((UINT) SmartEnergyControl[Which262].SmartIMin) << 15;
  SPIWriteInt|=((UINT) SmartEnergyControl[Which262].SmartDownStep)  << 13;
  SPIWriteInt|=((UINT) SmartEnergyControl[Which262].SmartStallLevelMax) << 8;
  SPIWriteInt|=((UINT) SmartEnergyControl[Which262].SmartUpStep) << 5;
  SPIWriteInt|=((UINT) SmartEnergyControl[Which262].SmartStallLevelMin);

  ReadWrite262(Which262, &SPIReadInt, SPIWriteInt);
  SPISmartConf=SPIWriteInt;
}


/*******************************************************************
   Funktion: WriteStallGuardConfig()
   Parameter: Which262: Auswahl des TMC262 bei Modulen mit mehr als 1 Achse

   Rckgabewert: ---

   Zweck: Beschreiben des SGCSCONF-Registers des TMC262.
          Die Parameter mssen in der globalen Variablen StallGuardConfig
          stehen.
********************************************************************/
static void WriteStallGuardConfig(UCHAR Which262)
{
  if(StallGuardConfig[Which262].StallGuardThreshold>63) StallGuardConfig[Which262].StallGuardThreshold=63;
  if(StallGuardConfig[Which262].StallGuardThreshold<-63) StallGuardConfig[Which262].StallGuardThreshold=-63;
  if(StallGuardConfig[Which262].CurrentScale>31) StallGuardConfig[Which262].CurrentScale=31;

  SPIWriteInt=0;
  SPIWriteInt|=BIT19|BIT18;  //Registeradresse SGSCONF
  if(StallGuardConfig[Which262].FilterEnable==1) SPIWriteInt|=BIT16;
  SPIWriteInt|=((UINT) StallGuardConfig[Which262].StallGuardThreshold & 0x7f) << 8;
  SPIWriteInt|=((UINT) StallGuardConfig[Which262].CurrentScale);

  ReadWrite262(Which262, &SPIReadInt, SPIWriteInt);
  SPISGConf=SPIWriteInt;
}


/*******************************************************************
   Funktion: WriteDriverConfig()
   Parameter: Which262: Auswahl des TMC262 bei Modulen mit mehr als 1 Achse

   Rckgabewert: ---

   Zweck: Beschreiben des DRVCONF-Registers des TMC262.
          Die Parameter mssen in der globalen Variablen DriverConfig
          stehen.
********************************************************************/
static void WriteDriverConfig(UCHAR Which262)
{
  SPIWriteInt=0;
  SPIWriteInt|=BIT19|BIT18|BIT17;  //Registeradresse DRVCONF
  SPIWriteInt|=((UINT) DriverConfig[Which262].SlopeHighSide & 0x03) << 14;
  SPIWriteInt|=((UINT) DriverConfig[Which262].SlopeLowSide & 0x03) << 12;
  if(DriverConfig[Which262].ProtectionDisable==1) SPIWriteInt|=BIT10;
  SPIWriteInt|=((UINT) DriverConfig[Which262].ProtectionTimer & 0x03) << 8;
  if(DriverConfig[Which262].StepDirectionDisable==1) SPIWriteInt|=BIT7;
  if(DriverConfig[Which262].VSenseScale==1) SPIWriteInt|=BIT6;
  SPIWriteInt|=((UINT) DriverConfig[Which262].ReadBackSelect & 0x03) << 4;
  if(DriverConfig[Which262].TestMode==1) SPIWriteInt|=BIT16;

  ReadWrite262(Which262, &SPIReadInt, SPIWriteInt);
  SPIDriverConf=SPIWriteInt;
}


/*******************************************************************
   Funktion: InitMotorDriver()
   Parameter: ---

   Rckgabewert: ---

   Zweck: Initialisierung des TMC259/TMC262 und der Variablen
********************************************************************/
void InitMotorDrivers(void)
{
  UINT i;

  //Datenstrukturen intialisieren
  for(i=0; i<N_O_MOTORS; i++)
  {
    //Parameter "non cool"
    StallGuardConfig[i].FilterEnable=1;
    StallGuardConfig[i].StallGuardThreshold=5;
    StallGuardConfig[i].CurrentScale=16;

    DriverConfig[i].SlopeHighSide=3;
    DriverConfig[i].SlopeLowSide=3;
    DriverConfig[i].ProtectionDisable=0;
    DriverConfig[i].ProtectionTimer=0;
    DriverConfig[i].StepDirectionDisable=0;
    DriverConfig[i].VSenseScale=1;
    DriverConfig[i].ReadBackSelect=TMC262_RB_SMART_ENERGY;

    SmartEnergyControl[i].SmartIMin=0;
    SmartEnergyControl[i].SmartDownStep=0;
    SmartEnergyControl[i].SmartStallLevelMax=0;
    SmartEnergyControl[i].SmartUpStep=0;
    SmartEnergyControl[i].SmartStallLevelMin=0;

    StepDirConfig[i].Intpol=0;
    StepDirConfig[i].DEdge=0;
    StepDirConfig[i].MRes=0;

    ChopperConfig[i].BlankTime=2;
    ChopperConfig[i].ChopperMode=0;
    ChopperConfig[i].HysteresisDecay=0;
    ChopperConfig[i].HysteresisEnd=2;
    ChopperConfig[i].HysteresisStart=3;
    ChopperConfig[i].TOff=5;
    ChopperConfig[i].DisableFlag=FALSE;
  }

  ENABLE_DRIVERS();

  //Initialwerte zum TMC262 senden
  for(i=0; i<N_O_MOTORS; i++)
  {
    WriteSmartEnergyControl(i);
    WriteStallGuardConfig(i);
    WriteDriverConfig(i);
    WriteStepDirConfig(i);
    WriteChopperConfig(i);
  }
}


void Set262StepDirMStepRes(UCHAR Which262, UCHAR MicrostepResolution)
{
  StepDirConfig[Which262].MRes=MicrostepResolution;
  if(MicrostepResolution!=4) StepDirConfig[Which262].Intpol=0;

  WriteStepDirConfig(Which262);
}

void Set262StepDirInterpolation(UCHAR Which262, UCHAR Interpolation)
{
  StepDirConfig[Which262].Intpol=Interpolation;
  if(Interpolation) StepDirConfig[Which262].MRes=4;

  WriteStepDirConfig(Which262);
}

void Set262StepDirDoubleEdge(UCHAR Which262, UCHAR DoubleEdge)
{
  StepDirConfig[Which262].DEdge=DoubleEdge;

  WriteStepDirConfig(Which262);
}

UCHAR Get262StepDirMStepRes(UCHAR Which262)
{
  return StepDirConfig[Which262].MRes;
}

UCHAR Get262StepDirInterpolation(UCHAR Which262)
{
  return StepDirConfig[Which262].Intpol;
}

UCHAR Get262StepDirDoubleEdge(UCHAR Which262)
{
  return StepDirConfig[Which262].DEdge;
}


void Set262ChopperBlankTime(UCHAR Which262, UCHAR BlankTime)
{
  ChopperConfig[Which262].BlankTime=BlankTime;
  WriteChopperConfig(Which262);
}

void Set262ChopperMode(UCHAR Which262, UCHAR Mode)
{
  ChopperConfig[Which262].ChopperMode=Mode;
  WriteChopperConfig(Which262);
}

void Set262ChopperRandomTOff(UCHAR Which262, UCHAR RandomTOff)
{
  ChopperConfig[Which262].RandomTOff=RandomTOff;
  WriteChopperConfig(Which262);
}

void Set262ChopperHysteresisDecay(UCHAR Which262, UCHAR HysteresisDecay)
{
  ChopperConfig[Which262].HysteresisDecay=HysteresisDecay;
  WriteChopperConfig(Which262);
}

void Set262ChopperHysteresisEnd(UCHAR Which262, UCHAR HysteresisEnd)
{
  ChopperConfig[Which262].HysteresisEnd=HysteresisEnd;
  WriteChopperConfig(Which262);
}

void Set262ChopperHysteresisStart(UCHAR Which262, UCHAR HysteresisStart)
{
  ChopperConfig[Which262].HysteresisStart=HysteresisStart;
  WriteChopperConfig(Which262);
}

void Set262ChopperTOff(UCHAR Which262, UCHAR TOff)
{
  ChopperConfig[Which262].TOff=TOff;
  WriteChopperConfig(Which262);
}

UCHAR Get262ChopperBlankTime(UCHAR Which262)
{
  return ChopperConfig[Which262].BlankTime;
}

UCHAR Get262ChopperMode(UCHAR Which262)
{
  return ChopperConfig[Which262].ChopperMode;
}

UCHAR Get262ChopperRandomTOff(UCHAR Which262)
{
  return ChopperConfig[Which262].RandomTOff;
}

UCHAR Get262ChopperHysteresisDecay(UCHAR Which262)
{
  return ChopperConfig[Which262].HysteresisDecay;
}

UCHAR Get262ChopperHysteresisEnd(UCHAR Which262)
{
  return ChopperConfig[Which262].HysteresisEnd;
}

UCHAR Get262ChopperHysteresisStart(UCHAR Which262)
{
  return ChopperConfig[Which262].HysteresisStart;
}

UCHAR Get262ChopperTOff(UCHAR Which262)
{
  return ChopperConfig[Which262].TOff;
}


void Set262SmartEnergyIMin(UCHAR Which262, UCHAR SmartIMin)
{
  SmartEnergyControl[Which262].SmartIMin=SmartIMin;
  WriteSmartEnergyControl(Which262);
}

void Set262SmartEnergyDownStep(UCHAR Which262, UCHAR SmartDownStep)
{
  SmartEnergyControl[Which262].SmartDownStep=SmartDownStep;
  WriteSmartEnergyControl(Which262);
}

void Set262SmartEnergyStallLevelMax(UCHAR Which262, UCHAR StallLevelMax)
{
  SmartEnergyControl[Which262].SmartStallLevelMax=StallLevelMax;
  WriteSmartEnergyControl(Which262);
}

void Set262SmartEnergyUpStep(UCHAR Which262, UCHAR SmartUpStep)
{
  SmartEnergyControl[Which262].SmartUpStep=SmartUpStep;
  WriteSmartEnergyControl(Which262);
}

void Set262SmartEnergyStallLevelMin(UCHAR Which262, UCHAR StallLevelMin)
{
  SmartEnergyControl[Which262].SmartStallLevelMin=StallLevelMin;
  WriteSmartEnergyControl(Which262);
}

UCHAR Get262SmartEnergyIMin(UCHAR Which262)
{
  return SmartEnergyControl[Which262].SmartIMin;
}

UCHAR Get262SmartEnergyDownStep(UCHAR Which262)
{
  return SmartEnergyControl[Which262].SmartDownStep;
}

UCHAR Get262SmartEnergyStallLevelMax(UCHAR Which262)
{
  return SmartEnergyControl[Which262].SmartStallLevelMax;
}

UCHAR Get262SmartEnergyUpStep(UCHAR Which262)
{
  return SmartEnergyControl[Which262].SmartUpStep;
}

UCHAR Get262SmartEnergyStallLevelMin(UCHAR Which262)
{
  return SmartEnergyControl[Which262].SmartStallLevelMin;
}


void Set262StallGuardFilter(UCHAR Which262, UCHAR Enable)
{
  StallGuardConfig[Which262].FilterEnable=Enable;
  WriteStallGuardConfig(Which262);
}

void Set262StallGuardThreshold(UCHAR Which262, signed char Threshold)
{
  StallGuardConfig[Which262].StallGuardThreshold=Threshold;
  WriteStallGuardConfig(Which262);
}

void Set262StallGuardCurrentScale(UCHAR Which262, UCHAR CurrentScale)
{
  StallGuardConfig[Which262].CurrentScale=CurrentScale;
  WriteStallGuardConfig(Which262);
}

UCHAR Get262StallGuardFilter(UCHAR Which262)
{
  return StallGuardConfig[Which262].FilterEnable;
}

signed char Get262StallGuardThreshold(UCHAR Which262)
{
  return StallGuardConfig[Which262].StallGuardThreshold;
}

UCHAR Get262StallGuardCurrentScale(UCHAR Which262)
{
  return StallGuardConfig[Which262].CurrentScale;
}


void Set262DriverSlopeHighSide(UCHAR Which262, UCHAR SlopeHighSide)
{
  DriverConfig[Which262].SlopeHighSide=SlopeHighSide;
  WriteDriverConfig(Which262);
}

void Set262DriverSlopeLowSide(UCHAR Which262, UCHAR SlopeLowSide)
{
  DriverConfig[Which262].SlopeLowSide=SlopeLowSide;
  WriteDriverConfig(Which262);
}

void Set262DriverDisableProtection(UCHAR Which262, UCHAR DisableProtection)
{
  DriverConfig[Which262].ProtectionDisable=DisableProtection;
  WriteDriverConfig(Which262);
}

void Set262DriverProtectionTimer(UCHAR Which262, UCHAR ProtectionTimer)
{
  DriverConfig[Which262].ProtectionTimer=ProtectionTimer;
  WriteDriverConfig(Which262);
}

void Set262DriverStepDirectionOff(UCHAR Which262, UCHAR SDOff)
{
  DriverConfig[Which262].StepDirectionDisable=SDOff;
  WriteDriverConfig(Which262);
}

void Set262DriverVSenseScale(UCHAR Which262, UCHAR Scale)
{
  DriverConfig[Which262].VSenseScale=Scale;
  WriteDriverConfig(Which262);
}

void Set262DriverReadSelect(UCHAR Which262, UCHAR ReadSelect)
{
  DriverConfig[Which262].ReadBackSelect=ReadSelect;
  WriteDriverConfig(Which262);
}

void Set262DriverTestMode(UCHAR Which262, UCHAR TestMode)
{
  DriverConfig[Which262].TestMode=TestMode;
  WriteDriverConfig(Which262);
}

UCHAR Get262DriverSlopeHighSide(UCHAR Which262)
{
  return DriverConfig[Which262].SlopeHighSide;
}

UCHAR Get262DriverSlopeLowSide(UCHAR Which262)
{
  return DriverConfig[Which262].SlopeLowSide;
}

UCHAR Get262DriverDisableProtection(UCHAR Which262)
{
  return DriverConfig[Which262].ProtectionDisable;
}

UCHAR Get262DriverProtectionTimer(UCHAR Which262)
{
  return DriverConfig[Which262].ProtectionTimer;
}

UCHAR Get262DriverStepDirectionOff(UCHAR Which262)
{
  return DriverConfig[Which262].StepDirectionDisable;
}

UCHAR Get262DriverVSenseScale(UCHAR Which262)
{
  return DriverConfig[Which262].VSenseScale;
}

UCHAR Get262DriverReadSelect(UCHAR Which262)
{
  return DriverConfig[Which262].ReadBackSelect;
}

UCHAR Get262DriverTestMode(UCHAR Which262)
{
  return DriverConfig[Which262].TestMode;
}

/*******************************************************************
   Funktion: Read262State()
   Parameter: Which262: Achsenummer
              Phases: Zeiger auf Variable fr Zustand der Phasenbits
              MStep: Zeiger auf Variable fr Mikroschrittposition
              StallGuard: Zeiger auf Variable fr StallGuard
              SmartEnergy: Zeiger auf Variable fr SmartEnergy
              Flags: Zeiger auf Variable fr Fehlerflags

   Rckgabewert: ---

   Zweck: Auslesen des Zustands TMC259/TMC262. Abhngig von RDSEL werden
          die zurckgelesenen Werte entsprechend aus dem SPI-Telegramm
          extrahiert.
          Fr Werte, die nicht bentigt werden, kann NULL eingesetzt
          werden.
********************************************************************/
void Read262State(UCHAR Which262, UCHAR *Phases, UCHAR *MStep, UINT *StallGuard, UCHAR *SmartEnergy, UCHAR *Flags)
{
  if(!IS_SD_MODE_SELECTED() && Which262==0)
  {
    if(!WaitingForDatagram)
    {
      Write428Zero(IDX_LOW_WORD);
      while(!(Read428Status() & TMC428_STATUS_CDGW));
      WaitingForDatagram=TRUE;
      return;
    }
    else
    {
      if((Read428Status() & TMC428_STATUS_CDGW)==0)
      {
        SPIReadInt=Read428Int(IDX_LOW_WORD);
        WaitingForDatagram=FALSE;
      }
      else return;
    }
  }

  if(IS_SD_MODE_SELECTED() || Which262!=0 || abs(GetSysTimer()-CoverDatagramDelay)>1000)
  {
    switch(ReadWriteConfig[Which262])
    {
      case RW_SDCONF:
        WriteStepDirConfig(Which262);
        ReadWriteConfig[Which262]=RW_CHOPCONF;
        break;

      case RW_CHOPCONF:
        WriteChopperConfig(Which262);
        ReadWriteConfig[Which262]=RW_SECTRL;
        break;

      case RW_SECTRL:
        WriteSmartEnergyControl(Which262);
        ReadWriteConfig[Which262]=RW_SGCONF;
        break;

      case RW_SGCONF:
        WriteStallGuardConfig(Which262);
        ReadWriteConfig[Which262]=RW_DRVCONF;
        break;

      case RW_DRVCONF:
        WriteDriverConfig(Which262);
        ReadWriteConfig[Which262]=RW_SDCONF;
        break;
    }
    CoverDatagramDelay=GetSysTimer();
  }

  switch(DriverConfig[Which262].ReadBackSelect)
  {
    case TMC262_RB_MSTEP:
      if(Phases!=NULL) *Phases=SPIReadInt >> 18;
      if(MStep!=NULL) *MStep=SPIReadInt >> 10;
      break;

    case TMC262_RB_STALL_GUARD:
      if(StallGuard!=NULL) *StallGuard=SPIReadInt >> 10;
      break;

    case TMC262_RB_SMART_ENERGY:
      if(StallGuard!=NULL)
      {
        *StallGuard=SPIReadInt >> 15;
        *StallGuard<<=5;
      }
      if(SmartEnergy!=NULL)
      {
        *SmartEnergy=(SPIReadInt >> 10) & 0x1f;
      }
      break;
  }

  if(Flags!=NULL) *Flags=SPIReadInt & 0xff;
}


/*******************************************************************
   Funktion: Disable262()
   Parameter: Which262: Achsenummer

   Rckgabewert: ---

   Zweck: Motor stromlos schalten. Dies wird erreicht durch Setzen
          der TOff-Zeit auf 0 (wobei natrlich der zuletzt mit
          Set262ChopperTOff() gesetzte Wert erhalten bleibt).
********************************************************************/
void Disable262(UCHAR Which262)
{
  if(!ChopperConfig[Which262].DisableFlag)
  {
    ChopperConfig[Which262].DisableFlag=TRUE;
    WriteChopperConfig(Which262);
  }
}


/*******************************************************************
   Funktion: Enable262()
   Parameter: Which262: Achsenummer

   Rckgabewert: ---

   Zweck: Motor einschalten. Dies wird erreicht durch Setzen
          der TOff-Zeit auf den zuletzt mit Set262ChopperTOff()
          eingestellten Wert.
********************************************************************/
void Enable262(UCHAR Which262)
{
  if(ChopperConfig[Which262].DisableFlag)
  {
    ChopperConfig[Which262].DisableFlag=FALSE;
    WriteChopperConfig(Which262);
  }
}


void GetSPIData(UCHAR Index, int *Data)
{
  switch(Index)
  {
    case 0:
      *Data=SPIReadInt;
      break;

    case 1:
      *Data=SPIStepDirConf;
      break;

    case 2:
      *Data=SPIChopperConf;
      break;

    case 3:
      *Data=SPISmartConf;
      break;

    case 4:
      *Data=SPISGConf;
      break;

    case 5:
      *Data=SPIDriverConf;
      break;

    case 6:
      *Data=ChopperConfig[0].DisableFlag;
      break;
  }
}
